#!/usr/bin/python

import sys

from PySide import QtGui
from PySide import QtDeclarative
import os
from os import mkdir, remove, stat, environ
from os.path import isfile, isdir, exists
import codecs
import dbus
# import python dbus GLib mainloop support
import dbus.mainloop.glib
# Enable glib main loop support
dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

# Comment the line below if you don't want to use OpenGL for QML rendering or if it is not supported
from PySide import QtOpenGL,  QtCore

from rss_sqlite import Listing
CONFIGDIR = environ.get("HOME", "/home/user") + "/.feedingit/"
#CONFIGDIR = "/home/user/.feedingit"

import logging
logger = logging.getLogger(__name__)

import debugging
debugging.init(dot_directory=".feedingit", program_name="feedingit-pyside")

from cgi import escape
from re import sub

class Controller(QtCore.QObject):
    
    def __init__(self, listing):
        QtCore.QObject.__init__(self)
        from XmlHandler import XmlHandler
        self._handler = XmlHandler(listing)
        
    def update_progress(self, percent_complete,
                        completed, in_progress, queued,
                        bytes_downloaded, bytes_updated, bytes_per_second,
                        feed_updated):
        logger.info(feed_updated)
        total = completed + in_progress + queued
        root.updateProgress(total, completed)
    
    def update_started(self):
        root.updateStarted()
    
    def update_finished(self):
        root.updateFinished()

    @QtCore.Slot(str,str, result=str)
    def getArticle(self, key, article):
       feed = listing.getFeed(key)
       try:
          file = codecs.open(feed.getContentLink(article), "r", "utf-8")
          html = file.read().replace("body", "body bgcolor='#ffffff'", 1)
          file.close()
       except:
          html = u"<html><body>Error retrieving article</body></html>"
       return html
    
    @QtCore.Slot(str, result=str)
    def getFeedsXml(self, catid):
        return self._handler.generateFeedsXml(catid)
    
    @QtCore.Slot(str,result=str)
    def getArticlesXml(self, key):
        #onlyUnread = arguments.get("onlyUnread","False")
        return self._handler.generateArticlesXml(key, config.getHideReadArticles())
    
    @QtCore.Slot(result=str)
    def getCategoryXml(self):
        return self._handler.generateCategoryXml()
    
    @QtCore.Slot(QtCore.QObject)
    def feedClicked(self, wrapper):
        #print 'User clicked on:', wrapper._key
        #articlesModel.updateModel(wrapper._key)
        pass
        
    @QtCore.Slot(str)
    def updateFeed(self, key):
        listing.updateFeed(key)
        
    @QtCore.Slot()
    def updateAll(self):
        for catid in listing.getListOfCategories():
            for feed in listing.getSortedListOfKeys("Manual", category=catid):
                listing.updateFeed(feed)

    @QtCore.Slot(str)
    def updateCategory(self, catid):
        for feed in listing.getSortedListOfKeys("Manual", category=catid):
            listing.updateFeed(feed)           

    @QtCore.Slot(str,str,str)
    def addFeed(self, title, url, catid):
        listing.addFeed(title,url, category=catid)
        
    @QtCore.Slot(str)
    def addCategory(self, name):
        listing.addCategory(name)

    @QtCore.Slot(str)
    def removeFeed(self, key):
        listing.removeFeed(key)

    @QtCore.Slot(str)
    def removeCategory(self, catid):
        listing.removeCategory(catid)

    @QtCore.Slot(str)
    def markAllAsRead(self, key):
        feed = listing.getFeed(key)
        feed.markAllAsRead()
        listing.updateUnread(key)

    @QtCore.Slot(str, str)
    def setEntryRead(self, key, articleid):
        feed = listing.getFeed(key)
        feed.setEntryRead(articleid)
        listing.updateUnread(key)

    @QtCore.Slot(str, result=str)
    def getConfig(self, item):
        if (item == "hideReadFeed"):
            return "True"
        if (item == "hideReadArticles"):
            return "False"
        return ""
    
    @QtCore.Slot(str, str)
    def populateFileDialog(self, path, type):
        import glob
        import os.path
        for file in glob.glob(path+type):
            logger.debug(file)
            root.addFileNotification(file, os.path.basename(file))
    
    @QtCore.Slot(str, result=int)
    def importOpml(self, filename):
        from opml_lib import parseOpml
        file = open(filename, "r")
        feeds = parseOpml(file.read())
        file.close()
        for (title, url) in feeds:
            listing.addFeed(title, url)
        return len(feeds)
    
    @QtCore.Slot(str, result=str)
    def exportOpml(self, filename="/home/user/MyDocs/feedingit-export.opml"):
        logger.debug("ExportOpmlData: %s" % filename)
        from opml_lib import getOpmlText
        try:
            str = getOpmlText(listing)
            file = open(filename, "w")
            file.write(str)
            file.close()
            return filename
        except:
            logger.debug("Error exporting: %s" % filename)
            return "error"
        
    @QtCore.Slot(str, result=bool)
    def getBooleanSetting(self, setting):
        if (setting == "theme"):
            return config.getTheme()
        elif (setting == "imageCache" ):
            return config.getImageCache()
        elif (setting == "hideReadFeeds"):
            return config.getHideReadFeeds()
        elif (setting == "hideReadArticles"):
            return config.getHideReadArticles()
        elif (setting == "autoupdate"):
            return config.isAutoUpdateEnabled()
        else:
            return 'True'
        
    @QtCore.Slot(str, bool)
    def setBooleanSetting(self, setting, value):
        if (setting == "theme"):
            config.setTheme(value)
        elif (setting == "imageCache" ):
            config.setImageCache(value)
        elif (setting == "hideReadFeeds"):
            config.setHideReadFeeds(value)
        elif (setting == "hideReadArticles"):
            config.setHideReadArticles(value)
        elif (setting == "autoupdate"):
            config.setAutoUpdateEnabled(value)
        config.saveConfig()

def main():

    if not isdir(CONFIGDIR):
        try:
            mkdir(CONFIGDIR)
        except:
            logger.error("Error: Can't create configuration directory")
            from sys import exit
            exit(1)
            
    from config import Config
    global config
    config = Config(None,CONFIGDIR+"config.ini")

    global listing
    listing = Listing(config, CONFIGDIR)
    
    import mainthread
    mainthread.init()

    from jobmanager import JobManager
    JobManager(True)

    app = QtGui.QApplication(sys.argv)
    view = QtDeclarative.QDeclarativeView()

    controller = Controller(listing)
 
    # listen on dbus for download update progress
    bus = dbus.SessionBus()
    bus.add_signal_receiver(handler_function=controller.update_progress,
                            bus_name=None,
                            signal_name='UpdateProgress',
                            dbus_interface='org.marcoz.feedingit',
                            path='/org/marcoz/feedingit/update')
    bus.add_signal_receiver(handler_function=controller.update_started,
                            bus_name=None,
                            signal_name='UpdateStarted',
                            dbus_interface='org.marcoz.feedingit',
                            path='/org/marcoz/feedingit/update')
    bus.add_signal_receiver(handler_function=controller.update_finished,
                            bus_name=None,
                            signal_name='UpdateFinished',
                            dbus_interface='org.marcoz.feedingit',
                            path='/org/marcoz/feedingit/update')
 
    global root
    rc = view.rootContext()
 
    rc.setContextProperty('controller', controller)

    # Comment the two lines below if you don't want to use OpenGL for QML rendering or if it is not supported
    #glw = QtOpenGL.QGLWidget()
    #view.setViewport(glw)

    if os.path.exists('/usr/share/feedingit/qml'):
        glw = QtOpenGL.QGLWidget()
        view.setViewport(glw)
        view.setSource('/usr/share/feedingit/qml/main.qml')
        view.showFullScreen()
    else:
        view.setSource(os.path.join('qml','main.qml'))
        view.show()
        #view.setSource(os.path.join('qml','FeedingIt.qml'))
    root = view.rootObject()

    #view.showFullScreen()
    #view.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    
    main()
